@@ -68,7 +68,6 @@ from deep_autoviml.modeling.one_cycle import OneCycleScheduler
 from sklearn.metrics import roc_auc_score, mean_squared_error, mean_absolute_error
 from IPython.core.display import Image, display
 import pickle
-#############################################################################################
 ##### Suppress all TF2 and TF1.x warnings ###################
 try:
     tf.logging.set_verbosity(tf.logging.ERROR)
@@ -94,12 +93,7 @@ from sklearn.metrics import balanced_accuracy_score, classification_report
 from sklearn.metrics import confusion_matrix, roc_auc_score, accuracy_score
 from collections import defaultdict
 from tensorflow.keras import callbacks
-#########################################################################################
-### This is the Storm-Tuner which stands for Stochastic Random Mutator tuner
-###  More details can be found in this github: https://github.com/ben-arnao/stochasticmutatortuner
-###   You can also pip install storm-tuner --upgrade to get the latest version ##########
-#########################################################################################
-from storm_tuner import Tuner
+#############################################################################################
 def train_image_model(deep_model, train_ds, valid_ds, cat_vocab_dict, 
                       keras_options, project_name, save_model_flag):
     epochs = check_keras_options(keras_options, "epochs", 20)
@@ -124,4 +118,4 @@ def train_image_model(deep_model, train_ds, valid_ds, cat_vocab_dict,
         print('     deep_autoviml image_model saved in %s directory' %save_model_path)
     else:
         print('\nModel not being saved since save_model_flag set to False...')
-    return deep_model, cat_vocab_dict
\ No newline at end of file
+    return deep_model, cat_vocab_dict
